/* ------------------------------------------------------------------------------------------------ */
/* FILE :        Abassi.h																			*/
/*																									*/
/* CONTENTS :																						*/
/*               All typedef, definitions and function prototypes for the Abassi RTOS				*/
/*																									*/
/*																									*/
/* Copyright (c) 2011-2012, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*  $Revision: 1.171 $																				*/
/*	$Date: 2012/06/28 15:19:52 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#ifndef __ABASSI_H__							/* To protecdt against re-include					*/
#define __ABASSI_H__

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#if !(defined(__KEIL__) && defined(__C51__))	/* Keil C51 does not have stdint.h. What is used	*/
  #include <stdint.h>							/* from it will be patched later on					*/
#endif

#ifdef OS_TEST_HOOK								/* All test RTOS build configurations are there		*/
  #include "AbassiTestCfg.h"
#endif

#ifdef OS_DEMO									/* All demo RTOS build configurations are there		*/
  #include "AbassiDemo.h"
#endif

#ifndef OS_DEF_IN_MAKE							/* If the following build options are used			*/
	#define OS_ALLOC_SIZE		0				/* When !=0, RTOS supplied OSalloc					*/
	#define OS_COOPERATIVE		0				/* When !=0, the  kernel is in cooperative mode		*/
	#define OS_EVENTS			0				/* != 0 when event flags are supported				*/
	#define OS_FCFS				0				/* Allow the use of 1st come 1st serve services		*/
	#define OS_IDLE_STACK		0				/* If IdleTask supplied & if so, stack size			*/
	#define OS_LOGGING_TYPE		0				/* Type of logging to use							*/
	#define OS_MAILBOX			0				/* != 0 when mailboxes are used						*/
	#define OS_MAX_PEND_RQST	0				/* Maximum number of requests performed in ISRs		*/
	#define OS_MTX_DEADLOCK		0				/* != 0 to enable mutex deadlock protection			*/
	#define OS_MTX_INVERSION	0				/* >0 priority inheritance, <0 priority ceiling		*/
	#define OS_NAMES			0				/* != 0 when named Tasks / Semaphores / Mailboxes	*/
	#define OS_NESTED_INTS		0				/* != 0 operating with nested interrupts			*/
	#define OS_PRIO_CHANGE		0				/* If a task priority can be changed at run time	*/
	#define OS_PRIO_MIN			0				/* Max priority, Idle = OS_PRIO_MIN, AdameEve = 0	*/
	#define OS_PRIO_SAME		0				/* Support multiple tasks with the same priority	*/
	#define OS_ROUND_ROBIN		0				/* Use round-robin, value specifies period in uS	*/
	#define OS_RUNTIME			0				/* If create Task / Semaphore / Mailbox at run time	*/
	#define OS_SEARCH_ALGO		0				/* If using a fast search							*/
	#define OS_STARVE_PRIO		0				/* Priority threshold for starving protection		*/
	#define OS_STARVE_RUN_MAX	0				/* Maximum Timer Tick for starving protection		*/
	#define OS_STARVE_WAIT_MAX	0				/* Maximum time on hold for starving protection		*/
	#define OS_STATIC_BUF_MBX	0				/* when OS_STATIC_MBOX != 0, # of buffer element	*/
	#define OS_STATIC_MBX		0				/* If !=0 how many mailboxes						*/
	#define OS_STATIC_NAME		0				/* If named mailboxes/semaphore/task, size in char	*/
	#define OS_STATIC_SEM		0				/* If !=0 how many semaphores and mutexes			*/
	#define OS_STATIC_STACK		0				/* if !=0 number of bytes for all stacks			*/
	#define OS_STATIC_TASK		0				/* If !=0 how many tasks (excluding A&E and Idle)	*/
	#define OS_STATIC_TIM_SRV	0				/* If !=0 how many timer services					*/
	#define OS_TASK_SUSPEND		0				/* If a task can suspend another one				*/
	#define OS_TIMEOUT			0				/* !=0 enables blocking timeout						*/
	#define OS_TIMER_CB			0				/* !=0 gives the timer callback period				*/
	#define OS_TIMER_SRV		0				/* !=0 includes the timer services					*/
	#define OS_TIMER_US			0				/* !=0 enables timer & specifies the period in uS	*/
	#define OS_USE_TASK_ARG		0				/* If tasks have arguments							*/
#endif

#define OSputchar(x)	(putchar(x))			/* Character output for ASCII logging				*/

extern  int			       OSintCtrl(int State);/* Function that deals with both dint() and eint()	*/
#define OSdint()		  (OSintCtrl(0))		/* Could split into 2 functions but only a few		*/
#define OSeint(x)	((void)OSintCtrl(x))		/* CPU cycles are added on OSdint(), not OSeint()	*/

/* ------------------------------------------------------------------------------------------------	*/
/* Compiler & Target processor specifics											   				*/
/* Many defines could be set to default to the most common value but MISRA does not accept #undef	*/
/* So we use an overload definition (OX_XXX) instead of the original (OS_XXX).						*/
/* Things would be much simpler using #undef...														*/


/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M3 / CCS																				*/
/* ------------------------------------------------------------------------------------------------ */

#if defined(__TI_COMPILER_VERSION__) && defined(__TI_TMS470_V7M3__)
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NIVC) interrupt levels		*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick (interrupt # == -1)	*/
	#define OX_MEMSET_LIB			0			/* The function call is same as inline, keep inline	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		241			/* NVIC has 256 sources of interrupt, drop 15 1st	*/
	#endif
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__							/* Install the protection for multi-threading on	*/
	  #include <_lock.h>						/* the libraries									*/
	  void OS_ARMlock(void);
	  void OS_ARMunlock(void);
	  int G_OSmtxOK;							/* No need to set to 0 as BSS is zeroed				*/
	#endif

	#if defined(OS_CMSIS_RTOS)					/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__						/* CMSIS needs to know if kernel started			*/
		int G_OSstarted;						/* No need to set to 0 as BSS is zeroed				*/
	  #endif
	  extern int G_OSstarted;
	  #define OX_CMSIS_INIT() 					\
		do {									\
			G_OSstarted=1;						\
			TSKsetPrio(G_OStaskNow, 3);			\
		} while(0)
	#else
	  #define OX_CMSIS_INIT()
	#endif

	#define OX_DEVICE_INIT()					\
		do {									\
			_register_lock(&OS_ARMlock);		\
			_register_unlock(&OS_ARMunlock);	\
		} while(0);								\
		G_OSmtxOK = 1;							\
		OX_CMSIS_INIT()

	#define OX_EXTRA_FCT						\
	void OS_ARMlock(void) {						\
		if (G_OSmtxOK != 0) {					\
			MTXlock(G_OSmutex, -1);				\
		}										\
	}											\
	void OS_ARMunlock(void) {					\
		if (G_OSmtxOK != 0) {					\
			MTXunlock(G_OSmutex);				\
		}										\
	}

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M0 & M3 & M4 / GCC																	*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__)																				\
   && (defined(__ARM_ARCH_6M__) || defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__))
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NIVC) interrupt levels		*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick (interrupt # == -1)	*/
	#define OX_MEMSET_LIB			0			/* The function call is same as inline, keep inline	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		241			/* NVIC has 256 sources of interrupt, drop 15 1st	*/
	#endif
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#if defined(OS_CMSIS_RTOS)					/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__						/* CMSIS needs to know if kernel started			*/
		int G_OSstarted;						/* No need to set to 0 as BSS is zeroed				*/
	  #endif
	  extern int G_OSstarted;
	  #define OX_CMSIS_INIT() 					\
		do {									\
			G_OSstarted=1;						\
			TSKsetPrio(G_OStaskNow, 3);			\
		} while(0)
	#else
	  #define OX_CMSIS_INIT()
	#endif

	#if defined(OS_ATOLLIC_REENT)				/* For Atollic re-entrance, the RTOS deals 2 ways	*/
	  #if OS_ATOLLIC_REENT != 0					/* When +ve, the RTOS allows standard functions		*/
												/* by updating _impure_ptr.  When -ve, holds _reent	*/
		#include <reent.h>						/* but user must used the _xxxx_r() function		*/

	    #define OX_TASK_XTRA_FIELD	1

		#if OS_ATOLLIC_REENT > 0
		  #ifndef OS_CMSIS_RTOS
		  	#define OX_TASK_XTRA()														\
				do {																	\
					if (Fct != NULL) {													\
						PID->XtraData[0] = (intptr_t)OS_alloc(sizeof(struct _reent));	\
						_REENT_INIT_PTR((struct _reent *)PID->XtraData[0]);				\
					}																	\
					else {																\
						PID->XtraData[0] = (intptr_t)_impure_ptr;						\
					}																	\
				} while(0)
		  #else
		  	#define OX_TASK_XTRA()														\
				PID->XtraData[0] = (intptr_t)&(thread_def->Reent);						\
				_REENT_INIT_PTR((struct _reent *)PID->XtraData[0])
		  #endif	
		#endif

		#if defined(__ABASSI_C__)
			#include <envlock.h>
			#include <locale.h>
			#include <malloc.h>
			int G_OSmtxOK;						/* No need to set to 0 as BSS is zeroed				*/
		#endif

		#define OX_DEVICE_INIT()							\
			OX_CMSIS_INIT();								\
			G_OSmtxOK = 1

		#define OX_EXTRA_FCT								\
			void __malloc_lock(struct _reent *reent) {		\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}											\
			}												\
			void __malloc_unlock(struct _reent *reent) {	\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}											\
			}												\
			void __env_lock(struct _reent *reent) {			\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}											\
			}												\
			void __env_unlock(struct _reent *reent) {		\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}											\
			}												\
			void __tz_lock(void) {							\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}											\
			}												\
			void __tz_unlock(void) {						\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}											\
			}
	  #endif
	#endif

	#ifndef OX_DEVICE_INIT
	  #define OX_DEVICE_INIT()								\
		OX_CMSIS_INIT()
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M0 & M3 & M4 / IAR																	*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCARM__) && defined(__ARM_PROFILE_M__)
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NIVC) interrupt levels		*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick (interrupt # == -1)	*/
	#define OX_MEMSET_LIB			0			/* The function call is same as inline, keep inline	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		241			/* NVIC has 256 sources of interrupt, drop 15 1st	*/
	#endif
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__							/* In case the multi-threading hooks are used		*/
	  int G_OSmtxOK;							/* No need to set to 0 as BSS is zeroed				*/
	#endif

	#if defined(OS_CMSIS_RTOS)					/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__						/* CMSIS needs to know if kernel started			*/
		int G_OSstarted;						/* No need to set to 0 as BSS is zeroed				*/
	  #endif
	  extern int G_OSstarted;
	  #define OX_DEVICE_INIT() 					\
		do {									\
			G_OSstarted=1;						\
			G_OSmtxOK = 1;						\
			TSKsetPrio(G_OStaskNow, 3);			\
		} while(0)
	#else
	  #define OX_DEVICE_INIT() 					\
		G_OSmtxOK = 1
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M0 & M3 & M4 / Keil																	*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__CC_ARM)	   						/* Keil RealView ARM Compiler						*/
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NIVC) interrupt levels		*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick						*/
	#define OX_MEMSET_LIB			0			/* The function call is less efficient than inline	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		241			/* NVIC has 256 sources of interrupt, drop 15 1st	*/
	#endif
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST
	#define inline __inline						/* Keil uses __inline instead of inline				*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#if defined(OS_CMSIS_RTOS)					/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__						/* CMSIS needs to know if kernel started			*/
		int G_OSstarted;						/* No need to set to 0 as BSS is zeroed				*/
	  #endif
	  extern int G_OSstarted;
	  #define OX_DEVICE_INIT() 					\
		do {									\
			G_OSstarted=1;						\
			TSKsetPrio(G_OStaskNow, 3);			\
		} while(0)
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* AVR / GCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__AVR__)
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		-1			/* Reset is 0, 1 is first available					*/
	#define OX_MEMSET_LIB			1			/* The function call is same as inline, keep inline	*/
	#define OX_N_INTERRUPTS			34			/* 35 but Reset is not included in table			*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* AVR / IAR																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCAVR__)
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		-1			/* Reset is 0, 1 is first available					*/
	#define OX_MEMSET_LIB			1			/* The function call is same as inline, keep inline	*/
	#define OX_N_INTERRUPTS			34			/* 35 but Reset is not included in table			*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* AVR32A / AVR32 Studio																			*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__AVR32_AVR32A__)
	#define OS_AVR32A_INT_GRP		19			/* Number of interrupt groups for this device		*/
	#define OS_AVR32A_INT_LINE		10			/* Largest number of interrupt lines in a group		*/

	#define OX_NESTED_INTS			1			/* The AVR32 has 4 nested interrupt levels			*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OS_AVR32A_INTC_REG		0xFFFF0800	/* Base address of the INTC register bank			*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			0			/* The function call is same as inline, keep inline	*/
	#define OX_N_INTERRUPTS			(OS_AVR32A_INT_GRP*OS_AVR32A_INT_LINE)
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			0			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#define OSavr32aISRmap(_Grp, _Line)		(((_Grp)*OS_AVR32A_INT_LINE)+(_Line))
	#define OSavr32aISRprio(_Grp, _Prio)	(((int *)OS_AVR32A_INTC_REG)[_Grp]=G_OSavrPrio[_Prio])
	extern int G_OSavrPrio[4];

	#define OX_DEVICE_INIT()					\
		do {									\
			extern void OSavrSetEBVA(void);		\
			OSavrSetEBVA();						\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* AVR32 / IAR																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCAVR32__)
  #if (__CORE__ == __AVR32A__)
	#define OS_AVR32A_INT_GRP		19			/* Number of interrupt groups for this device		*/
	#define OS_AVR32A_INT_LINE		10			/* Largest number of interrupt lines in a group		*/
	#define OX_NESTED_INTS			1			/* The AVR32 has 4 nested interrupt levels			*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OS_AVR32A_INTC_REG		0xFFFF0800	/* Base address of the INTC register bank			*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			0			/* The function call is same as inline, keep inline	*/
	#define OX_N_INTERRUPTS			(OS_AVR32A_INT_GRP*OS_AVR32A_INT_LINE)
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#define OSavr32aISRmap(_Grp, _Line)			(((_Grp)*OS_AVR32A_INT_LINE)+(_Line))
	extern void OSavr32aISRprio(int Grp, int Prio);

  #else
	#error "Abassi: Unsupported processor/compiler"

  #endif

/* ------------------------------------------------------------------------------------------------ */
/* C28x / CCS																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif (defined(__TI_COMPILER_VERSION__) && defined(__TMS320C28XX__))
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				0			/* CCS / C28X does not zero the BSS at start-up		*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		0			/* Don't care here									*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than inline	*/
	#define OX_N_INTERRUPTS			0			/* Interrupts are not handled by the RTOS			*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef long OSalign_t;						/* Data type that will use the alignment size		*/

	extern void OSpostISR(void);				/* Not using G_OSisrTbl but a prologue & epilogue	*/
	extern void OSpreISR(void);					/* are needed to deal with kernel request in ISRs	*/

	#ifdef __ABASSI_C__							/* This code is added when in Abassi.c				*/
	  #include <_lock.h>
	  void OS_C28Xlock(void);
	  void OS_C28Xunlock(void);
	  int G_OSmtxOK = 0;
	#endif
												/* Device specific init: attach mutex to library	*/
	#define OX_DEVICE_INIT()					\
		do { _register_lock(&OS_C28Xlock);		\
		     _register_unlock(&OS_C28Xunlock);	\
		} while(0);								\
		G_OSmtxOK = 1
												/* Device specific fct: library mutex				*/
	#define OX_EXTRA_FCT						\
	void OS_C28Xlock(void) {					\
		if (G_OSmtxOK != 0) {					\
			MTXlock(G_OSmutex, -1);				\
		}										\
	}											\
	void OS_C28Xunlock(void) {					\
		if (G_OSmtxOK != 0) {					\
			MTXunlock(G_OSmutex);				\
		}										\
	}

/* ------------------------------------------------------------------------------------------------ */
/* COLDFIRE / IAR																					*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCCF__)
	#define OX_NESTED_INTS			1			/* CF has 8 nested interrupt levels					*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		-64			/* First ISR is #64									*/
	#define OX_MEMSET_LIB			0			/* The function call is same as inline, keep inline	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		192			/* CF has total of 256 but first 64 are reserved	*/
	#endif
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int32_t OSalign_t;					/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* PIC32 / MPLAB GCC																				*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__PIC32MX__)
	#include <plib.h>
	#define OX_NESTED_INTS			1			/* The PIC32 has 8 nested interrupt levels			*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			0			/* With -Os in-line is more compact					*/
	#define OX_N_INTERRUPTS			64			/* PIC32 has 64 sources of interrupt				*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* With -Os in-line is more compact					*/
	#define REENT_POST

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__							/* Set this pragma only once						*/
	  #pragma interrupt ISRdispatch single vector 0	/* Needed to configure the interrupt controller	*/
	#endif										/* in single vector mode							*/

	#define OX_DEVICE_INIT()					\
		do {									\
			OSdint();							\
			INTEnableSystemSingleVectoredInt();	\
			mINTSingleVectorRegistorSelect(0);	\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* MSP430 / CCS																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__TI_COMPILER_VERSION__) && defined(__MSP430__)
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is  a single operation	*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than inline	*/
	#define OX_N_INTERRUPTS			0			/* MSP430 table is in assembly file (16/32/64)		*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST
	typedef int OSalign_t;						/* Data type that will use the alignment size		*/

	extern void InitISRtbl(void);				/* ISR table initialization in assembly file		*/
	#ifdef __ABASSI_C__
	  #include <msp430.h>
	  #include <_lock.h>
	  void OS_MSPlock(void);
	  void OS_MSPunlock(void);
	  extern int _system_pre_init(void);
	  int G_OSmtxOK;							/* No need to set to 0 as BSS is zeroed				*/
	#endif
												/* Device specific init: attach mutex to library	*/
	#define OX_DEVICE_INIT()					\
		do { InitISRtbl();						\
		     _register_lock(&OS_MSPlock);		\
		     _register_unlock(&OS_MSPunlock);	\
		} while(0);								\
		G_OSmtxOK = 1;
												/* Device specific fct: bss zeroing & library mutex	*/
	#define OX_EXTRA_FCT							\
	int _system_pre_init(void) {					\
		extern int  __cinit__;						\
		extern char __bss__;						\
		extern char __end__;						\
	    if (&__cinit__ != (int *)-1) {				\
    	    char *cptr;								\
			WDTCTL = WDTPW + WDTHOLD;				\
	        for (cptr=&__bss__ ; cptr<=&__end__ ;){	\
    	        *cptr++ = 0;						\
        	}										\
			_register_lock(&_nop);					\
			_register_unlock(&_nop);				\
			WDTCTL = WDTPW;							\
    	}    										\
		return(1);									\
	}												\
	void OS_MSPlock(void) {							\
		if (G_OSmtxOK != 0) {						\
			MTXlock(G_OSmutex, -1);					\
		}											\
	}												\
	void OS_MSPunlock(void) {						\
		if (G_OSmtxOK != 0) {						\
			MTXunlock(G_OSmutex);					\
		}											\
	}


/* ------------------------------------------------------------------------------------------------ */
/* MSP430 / GCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__MSP430__)
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than inline	*/
	#define OX_N_INTERRUPTS			0			/* MSP430 table is in assembly file (16/32/64)		*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int OSalign_t;						/* Data type that will use the alignment size		*/

	extern void InitISRtbl(void);				/* ISR table initialization in assembly file		*/

	#define OX_DEVICE_INIT()					\
		do {									\
			InitISRtbl();						\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* MSP430 / IAR																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICC430__)
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			1			/* Most processors int read is a single operation	*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than inline	*/
	#define OX_N_INTERRUPTS			0			/* MSP430 table is in assembly file (16/32/64)		*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef int OSalign_t;						/* Data type that will use the alignment size		*/

	extern void InitISRtbl(void);				/* ISR table initialization in assembly file		*/

	#ifdef __ABASSI_C__							/* In case the multi-threading hooks are used		*/
	  int G_OSmtxOK;							/* No need to set to 0 as BSS is zeroed				*/
	#endif

	#define OX_DEVICE_INIT()					\
		do {									\
			InitISRtbl();						\
			G_OSmtxOK = 1;						\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* Z80 / SDCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(SDCC) && defined(SDCC_z80)
	#define OX_NESTED_INTS			0			/* Z80 ints are level, so can't use nested ints		*/
	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				0			/* When the BSS is zeroed at start-up				*/
	#define OX_INT_1_READ			0			/* Reading an int requires 2 read ops				*/
	#define OX_INT_1_READ			1			/* Most processors int read is single operation		*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than inline	*/
	#define OX_N_INTERRUPTS			8			/* Z80 has 8 sources of interrupt					*/
	#define OX_USE_INT_REGISTER		1			/* Keep track of ISR enable/disable with variable	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* 8051 / Keil																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__KEIL__)	&& defined(__C51__)		/* KEIL COMPILERS									*/
	#define OX_NESTED_INTS						/* The 8051 uses a 2 priority scheme for interrupts	*/
	#define OX_AE_STACK_SIZE		512

	#pragma REGPARMS							/* Use registers to pass arguments					*/
	typedef long intptr_t;						/* Keil: no stdint.h. Ptr are 3 bytes				*/

	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				1			/* When the BSS is zeroed at start-up				*/
	#define OS_CPU_FREQ				11059200	/* Sometimes needed									*/
	#define OX_INT_1_READ			0			/* Reading an int requires 2 read ops				*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			1
	#define OX_N_INTERRUPTS			7			/* 8051 has 7 interrupts (+1 extra as example)		*/
	#define OS_SERIAL_BAUD			19200
	#define OX_USE_INT_REGISTER		1
	#define OX_USE_STRLIB			1
	#define REENT_POST				reentrant	/* KEIL C51 needs to be told about re-entrance		*/

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

	#ifdef OS_IDLE_STACK						/* If not defined, the error message will occur		*/
		#if ((OS_IDLE_STACK)==0)
			#error "Abassi: For Keil/8051, the Idle Task MUST be created with OS_IDLE_STACK"
		#endif
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* 8051 / SDCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(SDCC) &&  defined(SDCC_mcs51)
	#if ((OS_STATIC_STACK)==0)					/* Does not work with malloc()						*/
	  #error "Abassi: 8051 with SDCC MUST use a static stack memory pool using OS_STATIC_STACK"
	  #error "Abassi: Must be sized to OS_STATIC_TASK*512 + 255"
	#endif

	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				0			/* When the BSS is zeroed at start-up				*/
	#define OX_ISR_TBL_OFFSET		0			/* G_OSisrTbl index = ISRnmb+OX_ISR_TBL_OFFSET		*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than inline	*/
	#define OX_N_INTERRUPTS			5			/* 8051 has 5 sources of interrupts					*/
	#define OX_USE_INT_REGISTER		0			/* If a variable is needed to track the IE state	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than inline	*/
	#define REENT_POST
	#define printf printf_small

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

#else
	#error "Abassi: Unsupported processor/compiler"

#endif

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */
/* DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU ARE SURE YOU KNOW WHAT YOU ARE DOING			*/
/* DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU ARE SURE YOU KNOW WHAT YOU ARE DOING			*/
/* DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU ARE SURE YOU KNOW WHAT YOU ARE DOING			*/
/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------------------------------ */
/* Build options validation																			*/
/* ------------------------------------------------------------------------------------------------ */

#ifndef OS_ALLOC_SIZE
  #error "Abassi: The build option OS_ALLOC_SIZE hasn't been defined"
#elif (OS_ALLOC_SIZE) < 0
  #error "Abassi: OS_ALLOC_SIZE has a negative value"  
#endif
#ifndef OS_COOPERATIVE
  #error "Abassi: The build option OS_COOPERATIVE hasn't been defined"
#endif
#ifndef OS_EVENTS
  #error "Abassi: The build option OS_EVENTS hasn't been defined"
#endif
#ifndef OS_FCFS
  #error "Abassi: The build option OS_FCFS hasn't been defined"
#endif
#ifndef OS_IDLE_STACK
  #error "Abassi: The build option OS_IDLE_STACK hasn't been defined"
#elif (OS_IDLE_STACK) < 0
  #error "Abassi: OS_IDLE_STACK has a negative value"
#endif
#ifndef OS_LOGGING_TYPE
  #error "Abassi: The build option OS_LOGGING_TYPE hasn't been defined"
#elif (OS_LOGGING_TYPE) < 0
  #error "Abassi: OS_LOGGING_TYPE is negative"
#endif
#ifndef OS_MAILBOX
  #error "Abassi: The build option OS_MAILBOX hasn't been defined"
#endif
#ifndef OS_MAX_PEND_RQST
  #error "Abassi: The build option OS_MAX_PEND_RQST hasn't been defined"
#elif ((OS_MAX_PEND_RQST) < 2U)
  #error "Abassi: OS_MAX_PEND_RQST must be > 1"
#endif
#ifndef OS_MTX_DEADLOCK
  #error "Abassi: The build option OS_MTX_DEADLOCK hasn't been defined"
#endif
#ifndef OS_MTX_INVERSION
  #error "Abassi: The build option OS_MTX_INVERSION hasn't been defined"
#endif
#ifndef OS_NAMES
  #error "Abassi: The build option OS_NAMES hasn't been defined"
#endif
#ifndef OS_NESTED_INTS
  #error "Abassi: The build option OS_NESTED_INTS hasn't been defined"
#endif
#ifndef OS_PRIO_CHANGE
  #error "Abassi: The build option OS_PRIO_CHANGE hasn't been defined"
#endif
#ifndef OS_PRIO_MIN
  #error "Abassi: The build option OS_PRIO_MIN hasn't been defined"
#elif (OS_PRIO_MIN) < 0
  #error "Abassi: OS_PRIO_MIN is negative"
#endif
#ifndef OS_PRIO_SAME
  #error "Abassi: The build option OS_PRIO_SAME hasn't been defined"
#endif
#ifndef OS_ROUND_ROBIN
  #error "Abassi: The build option OS_ROUND_ROBIN hasn't been defined"
#endif
#ifndef OS_RUNTIME
  #error "Abassi: The build option OS_RUNTIME hasn't been defined"
#endif
#ifndef OS_SEARCH_ALGO
  #error "Abassi: The build option OS_SEARCH_ALGO hasn't been defined"
#endif
#ifndef OS_STARVE_PRIO
  #error "Abassi: The build option OS_STARVE_PRIO hasn't been defined"
#endif
#ifndef OS_STARVE_RUN_MAX
  #error "Abassi: The build option OS_STARVE_RUN_MAX hasn't been defined"
#endif
#ifndef OS_STARVE_WAIT_MAX
  #error "Abassi: The build option OS_STARVE_WAIT_MAX hasn't been defined"
#endif
#ifndef OS_STATIC_BUF_MBX
  #error "Abassi: The build option OS_STATIC_BUF_MBX hasn't been defined"
#elif (OS_STATIC_BUF_MBX) < 0
  #error "Abassi: OS_STATIC_BUF_MBX is negative"
#endif
#ifndef OS_STATIC_MBX
  #error "Abassi: The build option OS_STATIC_MBX hasn't been defined"
#elif (OS_STATIC_MBX) < 0
  #error "Abassi: OS_STATIC_MBX is negative"
#endif
#ifndef OS_STATIC_NAME
  #error "Abassi: The build option OS_STATIC_NAME hasn't been defined"
#elif (OS_STATIC_NAME) < 0
  #error "Abassi: OS_STATIC_NAME is negative"
#endif
#ifndef OS_STATIC_SEM
  #error "Abassi: The build option OS_STATIC_SEM hasn't been defined"
#elif (OS_STATIC_SEM) < 0
  #error "Abassi: OS_STATIC_SEM is negative"
#endif
#ifndef OS_STATIC_STACK
  #error "Abassi: The build option OS_STATIC_STACK hasn't been defined"
#elif (OS_STATIC_STACK) < 0
  #error "Abassi: OS_STATIC_STACK is negative"
#endif
#ifndef OS_STATIC_TASK
  #error "Abassi: The build option OS_STATIC_TASK hasn't been defined"
#elif (OS_STATIC_TASK) < 0
  #error "Abassi: OS_STATIC_TASK is negative"
#endif
#ifndef OS_STATIC_TIM_SRV
  #error "Abassi: The build option OS_STATIC_TIM_SRV hasn't been defined"
#elif (OS_STATIC_TIM_SRV) < 0
  #error "Abassi: OS_STATIC_TASK is negative"
#endif
#ifndef OS_TASK_SUSPEND
  #error "Abassi: The build option OS_TASK_SUSPEND hasn't been defined"
#endif
#ifndef OS_TIMEOUT
  #error "Abassi: The build option OS_TIMEOUT hasn't been defined"
#endif
#ifndef OS_TIMER_CB
  #error "Abassi: The build option OS_TIMER_CB hasn't been defined"
#elif (OS_TIMER_CB) < 0
  #error "Abassi: OS_TIMER_CB is negative"
#endif
#ifndef OS_TIMER_SRV
  #error "Abassi: The build option OS_TIMER_SRV hasn't been defined"
#endif
#ifndef OS_TIMER_US
  #error "Abassi: The build option OS_TIMER_US hasn't been defined"
#elif (OS_TIMER_US) < 0
  #error "Abassi: OS_TIMER_US is negative"
#endif
#ifndef OS_USE_TASK_ARG
  #error "Abassi: The build option OS_USE_TASK_ARG hasn't been defined"
#endif

/* ------------------------------------------------------------------------------------------------ */
/* Internal overloading																				*/
/* ------------------------------------------------------------------------------------------------ */

#if OS_COOPERATIVE								/* When the kernel is to operate in a cooperative	*/
	#define OX_ROUND_ROBIN		0				/* Round robin cannot exist							*/
	#define OX_SEARCH_ALGO		(-1)			/* The search algorithm must use the linked list	*/
#endif


#if OS_TIMER_US									/* If the timer is used, a few human definitions   	*/
  #define OS_TICK_PER_SEC	((int)(((((long)(OS_TIMER_US))/2)+1000000L)       /((long)(OS_TIMER_US))))
  #define OS_MS_TO_TICK(x)	((int)(((((long)(OS_TIMER_US))/2)+((x)*   1000L)) /((long)(OS_TIMER_US))))
  #define OS_SEC_TO_TICK(x) ((int)(((((long)(OS_TIMER_US))/2)+((x)*1000000L)) /((long)(OS_TIMER_US))))
  #define OS_HMS_TO_TICK(h,m,s)	(OS_SEC_TO_TICK(((h)*60+(m))*60+(s)))
  #if (OS_ROUND_ROBIN) && ((OS_COOPERATIVE)==0)	/* Round Robin value must be >= Timer us			*/
	#if (OS_ROUND_ROBIN) < 0					/* Need to have the absolute value					*/
	  #define OS_RR_ABS 		(-(OS_ROUND_ROBIN))
	#else
	  #define OS_RR_ABS 		(OS_ROUND_ROBIN)
	#endif
    #if (OS_RR_ABS) < (OS_TIMER_US)				/* Force it == if too small							*/
	  #if (OS_ROUND_ROBIN) > 0
		#define OX_ROUND_ROBIN	(OS_TIME_US)
	  #else
		#define OX_ROUND_ROBIN	(-(OS_TIME_US))
	  #endif
	  #define OX_RR_ABS 		(OX_ROUND_ROBIN)
	#endif										/* Number of ticks in a round robin time slice		*/
	#ifndef OX_RR_ABS
	  #define OX_RR_ABS			(OS_RR_ABS)
	#endif
	#define OX_RR_TICK			((OX_RR_ABS)/(OS_TIMER_US))
  #endif
#else
  #if (OS_ROUND_ROBIN) && ((OS_COOPERATIVE)==0)
	#error "Abassi: OS_ROUND_ROBIN is true, but OS_TIMER_US is false"
	#error "Abassi: This means round robin is activated but the timer is not enable"
  #endif
  #if OS_STARVE_WAIT_MAX
	#error "Abassi: OS_STARVE_WAIT_MAX is true, but OS_TIMER_US is false"
	#error "Abassi: This means starvation priority boosting is activated but the timer is not enable"
  #endif
  #if (OS_TIMEOUT) > 0
	#error "Abassi: OS_TIMEOUT is true, but OS_TIMER_US is false"
	#error "Abassi: This means timeouts are activated but the timer is not enable"
  #endif
  #if OS_TIMER_CB
	#error "Abassi: OS_TIMER_CB is true, but OS_TIMER_US is false"
	#error "Abassi: This means a timer callback function is supplied but the timer is not enable"
  #endif
  #if OS_TIMER_SRV
	#error "Abassi: OS_TIMER_SRV is true, but OS_TIMER_US is false"
	#error "Abassi: This means the timer services are enable but the timer is not enable"
  #endif
#endif


#if (OS_STATIC_TASK) && ((OS_RUNTIME)==0)
  #error "Abassi: OS_STATIC_TASK is true when OS_RUNTIME is false"
  #error "Abassi: OS_STATIC_TASK can only be used if OS_RUNTIME is true"
#endif
#if (OS_STATIC_STACK) && ((OS_RUNTIME)==0)
  #error "Abassi: OS_STATIC_STACK is true when OS_RUNTIME is false"
  #error "Abassi: OS_STATIC_STACK can only be used if OS_RUNTIME is true"
#endif
#if (OS_STATIC_SEM) && ((OS_RUNTIME)==0)
  #error "Abassi: OS_STATIC_SEM is true when OS_RUNTIME is false"
  #error "Abassi: OS_STATIC_SEM can only be used if OS_RUNTIME is true"
#endif
#if (OS_STATIC_MBX) && ((OS_RUNTIME)==0) && (OS_MAILBOX)
  #error "Abassi: OS_STATIC_MBX is true when OS_RUNTIME is false"
  #error "Abassi: OS_STATIC_MBX can only be used if OS_RUNTIME is true"
#endif
#if (OS_STATIC_TIM_SRV) && ((OS_RUNTIME)==0) && (OS_TIMER_SRV)
  #error "Abassi: OS_STATIC_SEM is true when OS_RUNTIME is false"
  #error "Abassi: OS_STATIC_SEM can only be used if OS_RUNTIME is true"
#endif


#if (OS_LOGGING_TYPE)==1						/* For ASCII dump, need named descriptors			*/
  #define OX_NAMES				1				/* So override the definition						*/
#endif

#if (OS_TASK_SUSPEND) || (OS_MTX_DEADLOCK) || (OS_MTX_INVERSION)
  #define OX_HANDLE_MTX			1				/* These 3 build options require info about mutex	*/
#else
  #define OX_HANDLE_MTX			0
#endif

#if (OS_MTX_INVERSION) || (OS_STARVE_WAIT_MAX)
  #define OX_PRIO_CHANGE		1				/* These 3 build options require changing task prio	*/
#endif

#if ((OS_MAILBOX)==0)
  #define OX_STATIC_MBX			0
  #define OX_STATIC_BUF_MBX 	0
#else
  #if (OS_STATIC_BUF_MBX) && ((OS_STATIC_MBX)==0)
	#error "Abassi: OS_STATIC_BUF_MBX is non-zero (true), but OS_STATIC_MBX is zero (false)"
    #error "Abassi: OS_STATIC_BUF_MBX can only be used if OS_STATIC_MBX is true"
  #endif
#endif

#ifdef OX_NAMES
 #if (OX_NAMES) <= 0
  #define OX_STATIC_NAME		0				/* Names not supported, no need to reserve memory	*/
 #endif
#else
 #if (OS_NAMES) <= 0
  #define OX_STATIC_NAME		0				/* Names not supported, no need to reserve memory	*/
 #endif
#endif

#if (OS_STARVE_WAIT_MAX) && ((OS_STARVE_PRIO) == (OS_PRIO_MIN))
  #define OX_STARVE_WAIT_MAX	0				/* When set to min prio, is turned OFF				*/
#else
  #define OX_STARVE_WAIT_MAX	(OS_STARVE_WAIT_MAX)
#endif

#if ((OX_STARVE_WAIT_MAX) || ((OS_TIMER_US) && ((OS_ROUND_ROBIN) && ((OS_COOPERATIVE)==0))))
  #define OX_PRIO_SAME			1
#endif

#if OX_STARVE_WAIT_MAX
  #if (OS_STARVE_PRIO) < 0
	#if ((-(OS_STARVE_PRIO)) > (OS_PRIO_MIN)) || ((-(OS_STARVE_PRIO)) < 0)
	  #error "Abassi: OS_STARVE_PRIO is out of range"
	#endif
  #else
	#if ((OS_STARVE_PRIO) > (OS_PRIO_MIN)) || ((OS_STARVE_PRIO) < 0)
	  #error "Abassi: OS_STARVE_PRIO is out of range"
	#endif
  #endif
  #if OS_STARVE_RUN_MAX == 0
	#error "Abassi: With starvation enable, OS_STARVE_RUN_MAX cannot be set to zero"
	#error "Abassi: Starvation is enable because OS_STARVE_WAIT_MAX is non-zero"
  #endif
#else
	#define OX_STARVE_PRIO		0
	#define OX_STARVE_RUN_MAX	0
#endif

#if (OS_TIMER_SRV) == 0
  #define OX_STATIC_TIM_SRV 0
#endif

/* ------------------------------------------------------------------------------------------------ */
/* Overloading of build definitions																	*/
/* All build options are remapped to OX_XXX to avoid confusion when using the tokens				*/
/* From now on, all build options are OX_XXX														*/
/* ------------------------------------------------------------------------------------------------ */

#ifndef OX_ALLOC_SIZE
  #define OX_ALLOC_SIZE			(OS_ALLOC_SIZE)
#endif
#ifndef OX_COOPERATIVE
  #define OX_COOPERATIVE		(OS_COOPERATIVE)
#endif
#ifndef OX_EVENTS
  #define OX_EVENTS				(OS_EVENTS)
#endif
#ifndef OX_FCFS
  #define OX_FCFS				(OS_FCFS)
#endif
#ifndef OX_IDLE_STACK
  #define OX_IDLE_STACK			(OS_IDLE_STACK)
#endif
#ifndef OX_LOGGING_TYPE
  #define OX_LOGGING_TYPE		(OS_LOGGING_TYPE)
#endif
#ifndef OX_MAILBOX
  #define OX_MAILBOX			(OS_MAILBOX)
#endif
#ifndef OX_MAX_PEND_RQST
  #define OX_MAX_PEND_RQST		(OS_MAX_PEND_RQST)
#endif
#ifndef OX_MTX_DEADLOCK
  #define OX_MTX_DEADLOCK		(OS_MTX_DEADLOCK)
#endif
#ifndef OX_MTX_INVERSION
  #define OX_MTX_INVERSION		(OS_MTX_INVERSION)
#endif
#ifndef OX_N_INTERRUPTS
  #define OX_N_INTERRUPTS		(OS_N_INTERRUPTS)
#endif
#ifndef OX_NAMES
  #define OX_NAMES				(OS_NAMES)
#endif
#ifndef OX_NESTED_INTS
  #define OX_NESTED_INTS		(OS_NESTED_INTS)
#endif
#ifndef OX_PRIO_CHANGE
  #define OX_PRIO_CHANGE		(OS_PRIO_CHANGE)
#endif
#ifndef OX_PRIO_MIN
  #define OX_PRIO_MIN			(OS_PRIO_MIN)
#endif
#ifndef OX_PRIO_SAME
  #define OX_PRIO_SAME			(OS_PRIO_SAME)
#endif
#ifndef OX_ROUND_ROBIN
  #define OX_ROUND_ROBIN		(OS_ROUND_ROBIN)
#endif
#ifndef OX_RUNTIME
  #define OX_RUNTIME			(OS_RUNTIME)
#endif
#ifndef OX_SEARCH_ALGO
  #define OX_SEARCH_ALGO		(OS_SEARCH_ALGO)
#endif
#ifndef OX_STARVE_PRIO
  #define OX_STARVE_PRIO		(OS_STARVE_PRIO)
#endif
#ifndef OX_STARVE_RUN_MAX
  #define OX_STARVE_RUN_MAX		(OS_STARVE_RUN_MAX)
#endif
#ifndef OX_STATIC_BUF_MBX
  #define OX_STATIC_BUF_MBX		(OS_STATIC_BUF_MBX)
#endif
#ifndef OX_STATIC_MBX
  #define OX_STATIC_MBX			(OS_STATIC_MBX)
#endif
#ifndef OX_STATIC_NAME
  #define OX_STATIC_NAME		(OS_STATIC_NAME)
#endif
#ifndef OX_STATIC_SEM
  #define OX_STATIC_SEM			(OS_STATIC_SEM)
#endif
#ifndef OX_STATIC_STACK
  #define OX_STATIC_STACK		(OS_STATIC_STACK)
#endif
#ifndef OX_STATIC_TASK
  #define OX_STATIC_TASK		(OS_STATIC_TASK)
#endif
#ifndef OX_STATIC_TIM_SRV
  #define OX_STATIC_TIM_SRV		(OS_STATIC_TIM_SRV)
#endif
#ifndef OX_TASK_SUSPEND
  #define OX_TASK_SUSPEND		(OS_TASK_SUSPEND)
#endif
#ifndef OX_TASK_XTRA_FIELD
  #define OX_TASK_XTRA_FIELD	0
#endif
#ifndef OX_TIMEOUT
  #define OX_TIMEOUT			(OS_TIMEOUT)
#endif
#ifndef OX_TIMER_CB
  #define OX_TIMER_CB			(OS_TIMER_CB)
#endif
#ifndef OX_TIMER_SRV
  #define OX_TIMER_SRV			(OS_TIMER_SRV)
#endif
#ifndef OX_TIMER_US
  #define OX_TIMER_US			(OS_TIMER_US)
#endif
#ifndef OX_USE_TASK_ARG
  #define OX_USE_TASK_ARG		(OS_USE_TASK_ARG)
#endif

/* ------------------------------------------------------------------------------------------------ */
/* Data types & definitions																			*/
/* ------------------------------------------------------------------------------------------------ */

/* ------------------------------------------------	*/
/* Semaphore descriptor								*/

typedef struct _SEM_t {							/* Semaphore descriptor								*/
	struct _TSK_t *Blocked;						/* Linked list of tasks blocked by this semaphore	*/
	struct _TSK_t *MtxOwner;					/* Owner / locker of the mutex (when it a mutex)	*/
  #if OX_HANDLE_MTX
	struct _SEM_t *MtxNext;						/* The next Mutex owned by the same task			*/
  #endif
  #if OX_NAMES
	const char    *SemName;						/* Name of the semaphore							*/
	struct _SEM_t *SemaNext;					/* Next semaphore in system (when searching name)	*/
  #endif
	int   Value;								/* Semaphore counter								*/
  #if OX_FCFS									/* When 1st code 1st serve enable					*/
	int IsFCFS;									/* This indicates if it's a FCFS semaphore/mutex	*/
  #endif										/* and reduces code size							*/
  #if (OX_MTX_INVERSION) < 0					/* For priority ceiling, we need to attach a		*/
	int CeilPrio;								/* priority to the mutex							*/
  #endif
} SEM_t;

typedef SEM_t MTX_t;							/* Mutexes are simply semaphores					*/

#if ((OX_RUNTIME) >= 0)
#if OX_FCFS										/* Would have prefer to use .field=XXX but some		*/
	#define INIT_SEM_ISFCFS				,0		/* compilers do not support it. So has to revert	*/
#else											/* to the old K&R initialization scheme, which means*/
	#define INIT_SEM_ISFCFS						/* every value is better be be at the right spot.	*/
#endif											/* At least a test suite double check it's done OK	*/
#if OX_HANDLE_MTX
	#define INIT_SEM_MTXNEXT			,(SEM_t *)NULL
#else
	#define INIT_SEM_MTXNEXT
#endif
#if (OX_MTX_INVERSION) < 0
	#define INIT_MTX_CEILPRIO			,(OX_PRIO_MIN)
	#define INIT_SEM_CEILPRIO			,0
#else
	#define INIT_MTX_CEILPRIO
	#define INIT_SEM_CEILPRIO
#endif
#if OX_NAMES
	#define INIT_SEM_NAME(_Name)		,(_Name)
	#define INIT_SEM_SEMANEXT			,(SEM_t *)NULL
#else
	#define INIT_SEM_NAME(_Name)
	#define INIT_SEM_SEMANEXT
#endif

#define SEM_STATIC(__Var, __Name)		\
  static SEM_t __Var##__ = {			\
	 (TSK_t *)NULL						\
	,(TSK_t *)NULL						\
	INIT_SEM_MTXNEXT					\
	INIT_SEM_NAME(__Name)				\
	INIT_SEM_SEMANEXT					\
	,0									\
	INIT_SEM_ISFCFS						\
	INIT_SEM_CEILPRIO					\
  };									\
  SEM_t *(__Var) = &(__Var##__)



  #define MTX_STATIC(__Var, __Name)		\
  static MTX_t __Var##__ = {			\
	 (TSK_t *)NULL						\
	,(TSK_t *)NULL						\
	INIT_SEM_MTXNEXT					\
	INIT_SEM_NAME(__Name)				\
	INIT_SEM_SEMANEXT					\
	,1									\
	INIT_SEM_ISFCFS						\
	INIT_MTX_CEILPRIO					\
  };									\
  MTX_t *(__Var) = &(__Var##__)
#endif

/* ------------------------------------------------ */
/* Timer Services descriptor						*/

#if OX_TIMER_SRV

typedef void(*OStimerFct_t)(intptr_t Arg);

  typedef struct _TIM_t {
	struct _TIM_t *Next;						/* Next timer descriptor in the linked list			*/
	struct _TIM_t *Prev;						/* Previous timer descriptor in the linked list		*/
  #if OX_NAMES
	const char    *TimName;						/* Name of a timer service							*/
	struct _TIM_t *TimNext;						/* Next timer in the list (to search name)			*/
  #endif
	void          *Ptr;							/* Ptr to fct, data or RTOS component				*/
	void (*FctPtr)(intptr_t Arg);				/* *Ptr to fct call upon timeout					*/
	intptr_t       Arg;							/* When operation needs an argument					*/
	unsigned int   Ops;							/* Operation to perform								*/
	int            Period;						/* When != 0, period of a periodic timer			*/
	int            TickTime;					/* Tick value for timeout							*/
	int            ToAdd;						/* Increment, decrement to add to argument			*/
  } TIM_t;

 #if ((OX_RUNTIME) >= 0)
  #if OX_NAMES
	#define INIT_TIM_NAME(_Name)		,(&G_OSnoName[0])
	#define INIT_TIM_TIMNEXT			,(TIM_t *)NULL
  #else
	#define INIT_TIM_NAME(_Name)
	#define INIT_TIM_TIMNEXT
  #endif

  #define TIM_STATIC(__Var, __Name)		\
   static TIM_t __Var##__ = {			\
	  (TIM_t *)NULL						\
	, (TIM_t *)NULL						\
	INIT_TIM_NAME(_Name)				\
	INIT_TIM_TIMNEXT					\
	, (void *)NULL						\
	, (void *)NULL						\
	, (intptr_t)0						\
	, 0U								\
	, 0									\
	, 0									\
	, 0									\
   };									\
   TIM_t *(__Var) = &(__Var##__)
 #endif
#endif

/* ------------------------------------------------	*/
/* Task descriptor									*/

typedef struct _TSK_t {							/* Process descriptor								*/
	void       *Stack;							/* Stack pointer when not running (context save)	*/
  #if OX_TASK_XTRA_FIELD
	intptr_t    XtraData[OX_TASK_XTRA_FIELD];
  #endif
    SEM_t       Priv;							/* Private semaphore								*/
	SEM_t      *Blocker;						/* Semaphore blocking this task						*/
	struct _TSK_t *Next;						/* Next task blocked/ready to run					*/
	struct _TSK_t *Prev;						/* Previous task blocked/ready to run				*/
  #if OX_PRIO_SAME
	struct _TSK_t *Last;						/* Last ready with same priority: eliminate scan	*/
  #endif
  #if OX_NAMES
	const char    *TskName;
	struct _TSK_t *TaskNext;					/* Next task in the list (to search name)			*/
  #endif
  #if OX_HANDLE_MTX
	SEM_t *MyMutex;								/* Linked list of all mutex I'm the owner			*/
  #endif 
  #if OX_USE_TASK_ARG
	void *TaskArg;								/* Pointer to the argument of the task				*/
  #endif
  #if (OX_SEARCH_ALGO) < 0						/* When using link list instead of array			*/
	struct _TSK_t *PrioNext;					/* Task at the lower priority						*/
	struct _TSK_t *PrioPrev;					/* Task at the higher priority						*/
  #endif
  #if OX_STARVE_WAIT_MAX
	struct _TSK_t *StrvNext;					/* Next in linked list of ready to run task			*/
	struct _TSK_t *StrvPrev;					/* Previous in linked list of ready to run task		*/
	struct _TSK_t *StrvLast;					/* Last in the linked list of ready to run			*/
  #endif
  #if (OX_TIMEOUT) > 0
	struct _TSK_t *TimNext;						/* Next TSK in the timeout linked list				*/
	struct _TSK_t *TimPrev;						/* Previous TSK in the timeout linked list			*/
	int            TickTime;					/* Tick value for timeout							*/
  #endif
  #if (OX_TASK_SUSPEND) || ((OX_TIMEOUT) > 0)
	int  PrevRetVal;							/* Return value before getting suspended			*/
  #endif
	int         Prio;							/* Task's priority									*/
  #if OX_TASK_SUSPEND
	unsigned int SuspRqst;						/* Request to suspend the task						*/
  #endif
  #if OX_ROUND_ROBIN
	unsigned int RRcnt;							/* This counts how many timer tick the task has		*/
	#if OX_ROUND_ROBIN < 0						/* Variable round-robin time slice duration			*/
	  unsigned int RRmax;						/* Round-robin slice time max						*/
	#endif
  #endif										/* been running										*/
  #if OX_COOPERATIVE
	int Yield;
  #endif
  #if OX_MTX_INVERSION							/* To eliminate priority inversion, need to			*/
	volatile int OriPrio;						/* temporary increase the priority of the task		*/
  #endif										/* Need volatile as one port has problem without	*/
  #if OX_EVENTS
	unsigned int EventAcc;						/* Accumulation of the events bits					*/
	unsigned int EventRX;						/* Event bits received								*/
	unsigned int EventAND;						/* AND mask for the events to unblock				*/
	unsigned int EventOR;						/* OR mask for the events to unblock				*/
  #endif
  #if OX_MAILBOX
	intptr_t       MboxMsg;						/* Message to write when blocked on (MBX full)		*/
  #endif
  #if OX_STARVE_WAIT_MAX
	volatile int   StrvState;					/* If the starvation protection is active			*/
	int            StrvPrio;					/* Priority before starvation priority promotion	*/
	int            StrvTimOut;					/* Timer tick value to increase task's priority		*/
  #endif
  #if (OX_STARVE_PRIO) < 0
	int  StrvPrioMax;
  #endif
  #if (OX_STARVE_RUN_MAX) < 0
	int StrvRunMax;
  #endif
  #if (OX_STARVE_WAIT_MAX) < 0
	int StrvWaitMax;
  #endif
} TSK_t;

#if ((OX_RUNTIME) >= 0)

typedef struct {								/* When Data ptr are smaller size than function	ptr */
	TSK_t Tsk;									/* then we can't re-use a data pointer to hold the	*/
	void (*TskFct)(void);						/* address of the function attached to the task in	*/
} _sTSK_t;										/* anything in the task descriptor. e.g MSP430X		*/

#if OX_TASK_SUSPEND								/* Same here, no .field=XXX							*/
    #define INIT_TASK_SUSPRQST			,0U
#else
	#define INIT_TASK_SUSPRQST
#endif
#if OX_TASK_XTRA_FIELD
  #if OX_TASK_XTRA_FIELD   == 1
	#define INIT_TASK_XTRADATA			,{((intptr_t)0)}
  #elif OX_TASK_XTRA_FIELD == 2
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0)}
  #elif OX_TASK_XTRA_FIELD == 3
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0),((intptr_t)0)}
  #elif OX_TASK_XTRA_FIELD == 4
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0),((intptr_t)0),((intptr_t)0)}
  #elif OX_TASK_XTRA_FIELD == 5
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0),((intptr_t)0),((intptr_t)0),((intptr_t)0)}
  #endif
#else
	#define INIT_TASK_XTRADATA
#endif
#if OX_PRIO_SAME
	#define INIT_TASK_LAST				,(TSK_t *)NULL
#else
	#define INIT_TASK_LAST
#endif
#if (OX_TIMEOUT) > 0
	#define INIT_TASK_TIMNEXT			,(TSK_t *)NULL
	#define INIT_TASK_TIMPREV			,(TSK_t *)NULL
	#define INIT_TASK_TICKTIME			,0
#else
	#define INIT_TASK_TIMNEXT
	#define INIT_TASK_TIMPREV
	#define INIT_TASK_TICKTIME
#endif
#if (OX_TASK_SUSPEND) || ((OX_TIMEOUT) > 0)
	#define INIT_TASK_PREVRETVAL		,0
#else
	#define INIT_TASK_PREVRETVAL
#endif
#if OX_ROUND_ROBIN
	#define INIT_TASK_RRCNT				,0U
	#if OX_ROUND_ROBIN < 0
		#define INIT_TASK_RRMAX			,(unsigned int)(OX_RR_TICK)
	#else
		#define INIT_TASK_RRMAX
	#endif
#else
	#define INIT_TASK_RRCNT
	#define INIT_TASK_RRMAX
#endif
#if OX_COOPERATIVE
	#define	INIT_TASK_YIELD				,0
#else
	#define	INIT_TASK_YIELD
#endif
#if OX_NAMES
	#define INIT_TASK_NAME(_Name)		,(_Name)
	#define INIT_TASK_TASKNEXT			,(TSK_t *)NULL
#else
	#define INIT_TASK_NAME(_Name)
	#define INIT_TASK_TASKNEXT
#endif
#if OX_MTX_INVERSION
	#define INIT_TASK_ORIPRIO(_Prio)	,(_Prio)
#else
	#define INIT_TASK_ORIPRIO(_Prio)
#endif
#if OX_HANDLE_MTX
	#define INIT_TASK_MYMUTEX			,(SEM_t *)NULL
#else
	#define INIT_TASK_MYMUTEX
#endif 
#if OX_USE_TASK_ARG
	#define INIT_TASK_TASKARG			,(void *)NULL
#else
	#define INIT_TASK_TASKARG
#endif
#if OX_EVENTS
	#define INIT_TASK_EVENTRX			,0U
	#define INIT_TASK_EVENTAND			,0U
	#define INIT_TASK_EVENTOR			,0U
	#define INIT_TASK_EVENTACC			,0U
#else
	#define INIT_TASK_EVENTRX
	#define INIT_TASK_EVENTAND
	#define INIT_TASK_EVENTOR
	#define INIT_TASK_EVENTACC
#endif
#if OX_MAILBOX
	#define INIT_TASK_MBOXMSG			,(intptr_t)0
#else
	#define INIT_TASK_MBOXMSG
#endif
#if OX_STARVE_WAIT_MAX
	#define INIT_TASK_STARVNEXT			,(TSK_t *)NULL
	#define INIT_TASK_STARVPREV			,(TSK_t *)NULL
	#define INIT_TASK_STARVLAST			,(TSK_t *)NULL
	#define INIT_TASK_STARVSTATE		,0
	#define INIT_TASK_STARVPRIO			,0
	#define INIT_TASK_STARVTIMOUT		,0
	#if (OX_STARVE_PRIO) < 0
	  #define INIT_TASK_STARVPRIOMAX	,-(OX_STARVE_PRIO)
	#else
	  #define INIT_TASK_STARVPRIOMAX
	#endif
	#if (OX_STARVE_RUN_MAX) < 0
	  #define INIT_TASK_STARVRUNMAX		,-(OX_STARVE_RUN_MAX)
	#else
	  #define INIT_TASK_STARVRUNMAX
	#endif
	#if (OX_STARVE_WAIT_MAX) < 0
	  #define INIT_TASK_STARVWAITMAX	,-(OX_STARVE_WAIT_MAX)
	#else
	  #define INIT_TASK_STARVWAITMAX
	#endif
#else
	#define INIT_TASK_STARVNEXT
	#define INIT_TASK_STARVPREV
	#define INIT_TASK_STARVLAST
	#define INIT_TASK_STARVSTATE
	#define INIT_TASK_STARVPRIO
	#define INIT_TASK_STARVTIMOUT
	#define INIT_TASK_STARVPRIOMAX
	#define INIT_TASK_STARVRUNMAX
	#define INIT_TASK_STARVWAITMAX
#endif
#if (OX_SEARCH_ALGO) < 0						/* When using link list instead of array			*/
	#define INIT_TASK_PRIONEXT			,(TSK_t *)NULL
	#define INIT_TASK_PRIOPREV			,(TSK_t *)NULL
#else
	#define INIT_TASK_PRIONEXT
	#define INIT_TASK_PRIOPREV
#endif


#define TSK_STATIC(__Tsk, __Name, __Prio, __StackSize, __Fct)										\
  static OSalign_t __Tsk##__stack[((__StackSize)+sizeof(OSalign_t)-1)/sizeof(OSalign_t)];			\
  static _sTSK_t __Tsk##_s_ = {			\
  {										\
   (void *)&__Tsk##__stack[0]			\
	INIT_TASK_XTRADATA					\
  ,{									\
	 (TSK_t *)&__Tsk##_s_.Tsk			\
	,(TSK_t *)NULL						\
	INIT_SEM_MTXNEXT					\
	INIT_SEM_NAME(&G_OSprivName[0])		\
	INIT_SEM_SEMANEXT					\
	,((int)sizeof(OSalign_t)*(((__StackSize)+(int)sizeof(OSalign_t)-1)/(int)sizeof(OSalign_t)))		\
	INIT_SEM_ISFCFS						\
	INIT_SEM_CEILPRIO					\
  }										\
  ,(SEM_t *)&__Tsk##_s_.Tsk.Priv		\
  ,(TSK_t *)NULL						\
  ,(TSK_t *)NULL						\
  INIT_TASK_LAST						\
  INIT_TASK_NAME(__Name)				\
  INIT_TASK_TASKNEXT					\
  INIT_TASK_MYMUTEX						\
  INIT_TASK_TASKARG						\
  INIT_TASK_PRIONEXT					\
  INIT_TASK_PRIOPREV					\
  INIT_TASK_STARVNEXT					\
  INIT_TASK_STARVPREV					\
  INIT_TASK_STARVLAST					\
  INIT_TASK_TIMNEXT						\
  INIT_TASK_TIMPREV						\
  INIT_TASK_TICKTIME					\
  INIT_TASK_PREVRETVAL					\
  ,(__Prio)								\
  INIT_TASK_SUSPRQST					\
  INIT_TASK_RRCNT						\
  INIT_TASK_RRMAX						\
  INIT_TASK_YIELD						\
  INIT_TASK_ORIPRIO(__Prio)				\
  INIT_TASK_EVENTACC					\
  INIT_TASK_EVENTRX						\
  INIT_TASK_EVENTAND					\
  INIT_TASK_EVENTOR						\
  INIT_TASK_MBOXMSG						\
  INIT_TASK_STARVSTATE					\
  INIT_TASK_STARVPRIO					\
  INIT_TASK_STARVTIMOUT					\
  INIT_TASK_STARVPRIOMAX				\
  INIT_TASK_STARVRUNMAX					\
  INIT_TASK_STARVWAITMAX				\
  },									\
  (__Fct)								\
};										\
TSK_t *(__Tsk) = &__Tsk##_s_.Tsk
#endif

#ifndef __CC_ARM									/* Keil at -O2 & -O3 generates bad code			*/
  #define TSK_SETUP(_Tsk, _State)											\
	do {																	\
	(_Tsk)->Stack = OSstack((_Tsk)->Stack,									\
                          (_Tsk)->Priv.Value,								\
                          ((_sTSK_t *)(_Tsk))->TskFct);						\
	(_Tsk)->Priv.Value = 0;													\
	if ((_State) != 0) {													\
		(void)Abassi((ABASSI_POST), (void *)&((_Tsk)->Priv), (intptr_t)0);	\
	}} while (0)
#else
  #define TSK_SETUP(_Tsk, _State)											\
	do {																	\
	(_Tsk)->Stack	= OSstack((_Tsk)->Stack,								\
                          (_Tsk)->Priv.Value,								\
                          ((_sTSK_t *)(_Tsk))->TskFct);						\
	(_Tsk)->Priv.Value   = ((int)(_Tsk)->Next);	 /* repair */				\
	(_Tsk)->Priv.Blocked = (_Tsk);											\
	if ((_State) != 0) {													\
		(void)Abassi((ABASSI_POST), (void *)&((_Tsk)->Priv), (intptr_t)0);	\
	}} while(0)
#endif

/* ------------------------------------------------	*/
/* Mailbox descriptor								*/

#if OX_MAILBOX
  typedef struct _MBX_t {
	intptr_t *Buffer;							/* Holds the queue / mailbox elements				*/
  #if OX_NAMES									/* Name of the queue /  mailbox						*/
	const char    *MbxName;
	struct _MBX_t *MbxNext;
  #endif
	SEM_t     MySema;							/* Semaphore when no buffer in the mailbox			*/
	int       Size;								/* Number of elements in the mailbox				*/
	int       RdIdx;							/* Read index										*/
	int       WrtIdx;							/* Write index										*/
  } MBX_t;

 #if ((OX_RUNTIME) >= 0)						/* Same here, no .field=XXX							*/
  #if OX_NAMES
	#define INIT_MBX_NAME(_Name)		,(_Name)
	#define INIT_MBX_MBXNEXT			,(MBX_t *)NULL
  #else
	#define INIT_MBX_NAME(_Name)
	#define INIT_MBX_MBXNEXT
  #endif

  #define MBX_STATIC(__Var, __Name, __Size)		\
   static intptr_t __Var##__buf[(__Size)+1];	\
   MBX_t __Var##__ = {							\
   &(__Var##__buf[0])							\
   INIT_MBX_NAME(__Name)						\
   INIT_MBX_MBXNEXT								\
   , {											\
	 (TSK_t *)NULL								\
	,(TSK_t *)NULL								\
	INIT_SEM_MTXNEXT							\
	INIT_SEM_NAME(&G_OSnoName[0])				\
	INIT_SEM_SEMANEXT							\
	,0											\
	INIT_SEM_ISFCFS								\
	INIT_SEM_CEILPRIO							\
   }											\
   ,(__Size)									\
   ,0											\
   ,(__Size)									\
   };											\
   MBX_t *(__Var) = &(__Var##__)
 #endif
#endif

/* ------------------------------------------------	*/
/* Kernel requests bit masks						*/
													/* MISRA has issues with (1U<<XX)				*/
#define ABASSI_BIN         (0x0001U)				/* Binary semaphore & extra tag to some cmds	*/
#define ABASSI_WAIT        (0x0002U)				/* Regular semaphore waiting / mutex unblocking	*/
#define ABASSI_MTX         (0x0004U)				/* Mutex priority inheritance checking			*/
#define ABASSI_POST        (0x0008U)				/* Semaphore posting / mutex unblocking			*/
#define ABASSI_TOUT_RM     (0x0010U)				/* Remove from timeout linked list				*/
#define ABASSI_TOUT_ADD    (0x0020U)				/* Add in the timeout list						*/
#define ABASSI_TOUT_CHK    (0x0040U)				/* Check timer expiry's							*/
#define ABASSI_ADD_BLK     (0x0080U)				/* Add the task to the blocked list				*/
#define ABASSI_ADD_RUN     (0x0100U)				/* Add the task to the ready to run list		*/
#define ABASSI_NEW_PRIO    (0x0200U)				/* Set the task to a new priority				*/
#define ABASSI_EVENT       (0x0400U)				/* Event operation								*/
#define ABASSI_MBX_PUT     (0x0800U)				/* Put a mailbox message						*/
#define ABASSI_MBX_GET     (0x1000U)				/* Get a mailbox message						*/
#define ABASSI_RESUSP      (0x2000U)				/* Suspend or resume a task						*/
#define ABASSI_STARVE      (0x4000U)				/* Process starving (prio aging) protection		*/
#define ABASSI_TIM_SRV     (0x8000U)				/* Operation on timer services					*/

#define ABASSI_TIM_RM      (ABASSI_BIN)
#define ABASSI_TIM_ADD     (ABASSI_MTX)
#define ABASSI_TOUT_RM0    (ABASSI_BIN)				/* Remove from timeout list + zero TimedOut		*/
#define ABASSI_NEW_O_PRIO  (ABASSI_BIN)				/* New prio + set original						*/
#define ABASSI_SUSPEND     (ABASSI_BIN)				/* For ABASSI_RESUSP this means to suspend		*/

/* ------------------------------------------------	*/
/* Global variables									*/

extern volatile unsigned int G_OSstate[5];			/* If we are in an ISR or not and nesting level	*/
#if OX_N_INTERRUPTS
  extern        void (*G_OSisrTbl[OX_N_INTERRUPTS])(void) REENT_POST;	/* Table used by ISRdispatch*/
#else
  extern        void (*G_OSisrTbl[1])(void) REENT_POST;	/* Imported/sized in ASM file, dummy size	*/
#endif
extern const    char   G_OSprivName[5];				/* Name for a task's private semaphore			*/
extern const    char   G_OSnoName[4];				/* Name for un-named components					*/
extern          TSK_t *G_OStaskNow;					/* Currently running task						*/
extern volatile int    G_OStimCnt;					/* RTOS time base counter tick					*/
extern          MTX_t *G_OSmutex;					/* OS needs a mutex for some service creation	*/
													/* Made available even if RTOS does not use it	*/

/* ------------------------------------------------	*/
/* Atomic function prototypes						*/

extern int      Abassi      (unsigned int Ops, void *Ptr, intptr_t Arg) REENT_POST;
#if OX_IDLE_STACK
  extern void   IdleTask	(void);
#endif
#if (OX_MAILBOX) && (OX_RUNTIME)
  extern MBX_t *MBX__open   (const char *Name, int MbxSize, unsigned int IsFCFS) REENT_POST;
#endif
extern void    *OSalloc     (size_t MemSize);
extern void     OScontext   (TSK_t *NewTask);
extern void     OSinvalidISR(void) REENT_POST;
extern void    *OSstack     (void *StackBase, int StackSize, void (*TskFct)(void)) REENT_POST;
extern void     OSstart     (void);
#if OX_RUNTIME
  extern SEM_t *SEM__open   (const char *Name, unsigned int Type) REENT_POST;
  extern TSK_t *TSKcreate   (const char *Name, int TskPrio, int StackSize, void (*Fct)(void), int State);
#endif
extern void     TIMcallBack (void);
#if (OX_NAMES) && (OX_RUNTIME)
  extern TSK_t *TSKgetID_   (const char *Name) REENT_POST;
#endif
extern void     TSKstart    (void (*TaskFct)(void)) REENT_POST;


/* ------------------------------------------------	*/
/* Function defines									*/

#define OSisrInstall(Nmb, Fct)			(G_OSisrTbl[(Nmb)+(OX_ISR_TBL_OFFSET)]=(Fct))

/* ------------------------------------------------	*/
/* Events related macros							*/

#if OX_EVENTS
  #define EVTget()					(G_OStaskNow->EventRX)
  #define EVTgetAcc()				(G_OStaskNow->EventAcc)
  #define EVTreset()				(G_OStaskNow->EventRX=0U)
  #define EVTresetAcc()				(G_OStaskNow->EventAcc=0U)
  #define EVTset(_Tsk, _Bits)		((void)Abassi((ABASSI_EVENT),(void *)(_Tsk),(intptr_t)(_Bits)))
  #define EVTwait(_And,_Or,_Tout)	(G_OStaskNow->Priv.Value = 0,									\
									 G_OStaskNow->EventOR    = (_Or),								\
									 G_OStaskNow->EventAND   = ((_And) == 0U)						\
									                         ? (_Or)								\
									                         : (_And),								\
									 Abassi(((ABASSI_WAIT)|(ABASSI_EVENT)), 						\
										    (void *)&(G_OStaskNow->Priv), (intptr_t)(_Tout)))
#endif


/* ------------------------------------------------	*/
/* Mailboxes related macros							*/

#if OX_MAILBOX

	extern int MBXget(MBX_t *Mbx,     intptr_t *Msg, int Tout) REENT_POST;
	extern int MBXput(MBX_t *Mailbox, intptr_t  Msg, int Tout) REENT_POST;

  #if OX_RUNTIME
	#define MBXopen(_Tsk, _Size)		(MBX__open((_Tsk), (_Size), 0U))
   #if OX_FCFS
	#define MBXopenFCFS(_Tsk, _Size)	(MBX__open((_Tsk), (_Size), (ABASSI_BIN)))
   #endif
  #endif

  #if OX_FCFS
	#define MBXsetFCFS(_Mbox)			((_Mbox)->MySema.IsFCFS=1)
	#define MBXnotFCFS(_Mbox)			((_Mbox)->MySema.IsFCFS=0)
  #endif


#endif

/* ------------------------------------------------	*/
/* Mutexes related macros							*/

#if OX_RUNTIME
  #define MTXopen(_Name)		(SEM__open((_Name), 1U))
#endif
#define MTXowner(_Mtx)			((_Mtx)->MtxOwner)
#define MTXlock(_Mtx, _Tout)	(Abassi(((ABASSI_WAIT)|(ABASSI_MTX)|(ABASSI_BIN)), (void *)(_Mtx),	  \
								        (intptr_t)(_Tout)))
#define MTXunlock(_Mtx)			((void)Abassi(((ABASSI_POST)|(ABASSI_MTX)),(void *)(_Mtx),(intptr_t)0))
#if OX_FCFS
 #if OX_RUNTIME
  #define MTXopenFCFS(_Name)	(SEM__open((_Name), 3U))
 #endif
  #define MTXsetFCFS(_Mtx)		((_Mtx)->IsFCFS=1)
  #define MTXnotFCFS(_Mtx)		((_Mtx)->IsFCFS=0)
#endif
#if (OX_MTX_INVERSION) < 0
  #define MTXgetCeilPrio(_Mtx)			(((volatile MTX_t *)(_Mtx))->CeilPrio)
  #define MTXsetCeilPrio(_Mtx, _Prio)	((_Mtx)->CeilPrio=(_Prio))
#endif


/* ------------------------------------------------	*/
/* Semaphores related macros						*/

#if OX_RUNTIME
  #define SEMopen(_Name)		(SEM__open((_Name), 0U))
#endif
#define SEMpost(_Sem)			((void)Abassi((ABASSI_POST), (void *)(_Sem), (intptr_t)0))
#define SEMwait(_Sem, _Tout)	(Abassi((ABASSI_WAIT), (void *)(_Sem), (intptr_t)(_Tout)))
#define SEMwaitBin(_Sem, _Tout)	(Abassi(((ABASSI_WAIT)|(ABASSI_BIN)),(void *)(_Sem),(intptr_t)(_Tout)))
#if OX_FCFS
 #if OX_RUNTIME
  #define SEMopenFCFS(_Name)	(SEM__open((_Name), 2U))
 #endif
  #define SEMsetFCFS(_Sema)		((_Sema)->IsFCFS=1)
  #define SEMnotFCFS(_Sema)		((_Sema)->IsFCFS=0)
#endif


/* ------------------------------------------------	*/
/* Timer Services related macros					*/

#if OX_TIMER_SRV
  #if (OX_RUNTIME)
	extern  TIM_t *TIMopen(const char *Name) REENT_POST;
  #endif

	#define TIM_CALL_FCT		((0xFFFFU) & ~(ABASSI_TIM_SRV))
 	#define TIM_WRITE_DATA		((0xEEEEU) & ~(ABASSI_TIM_SRV))

	#define TIMarg(_Timer, _Arg)		((_Timer)->Arg   = (intptr_t)(_Arg))
	#define TIMtoAdd(_Timer, _ToAdd)	((_Timer)->ToAdd = (int)(_ToAdd))
	#define TIMkill(_Timer)				((void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_RM)),			\
										              (void *)(_Timer), (intptr_t)0))
	#define TIMperiod(_Timer, _Period)	((_Timer)->Period = (_Period))

	#define TIMdata(_Timer, _Addr, _Data, _ToAdd, _Expiry, _Period)									\
				do {																				\
					(_Timer)->Ops    = (TIM_WRITE_DATA) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Addr);												\
					(_Timer)->Arg    = (intptr_t)(_Data);											\
					(_Timer)->ToAdd  = (int)(_ToAdd);												\
					(_Timer)->Period = (int)(_Period); 												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)),(void *)( _Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

  #if OX_EVENTS
	#define TIMevt(_Timer, _Tsk, _Data, _Expiry, _Period)											\
				do {																				\
					(_Timer)->Ops    = (ABASSI_EVENT) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Tsk);												\
					(_Timer)->Arg    = (intptr_t)(_Data);											\
					(_Timer)->ToAdd  = 0;															\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)
  #endif

	#define TIMfct(_Timer, _Fct, _Arg, _ToAdd, _Expiry, _Period)									\
				do {																				\
					(_Timer)->Ops    = (TIM_CALL_FCT) | (ABASSI_TIM_SRV);							\
					(_Timer)->FctPtr = (_Fct);														\
					(_Timer)->Arg    = (intptr_t)(_Arg);											\
					(_Timer)->ToAdd  = (int)(_ToAdd);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

  #if OX_MAILBOX
	#define TIMmbx(_Timer, _Mbx, _Data, _ToAdd, _Expiry, _Period)									\
				do {																				\
					(_Timer)->Ops    = (ABASSI_MBX_PUT) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Mbx);												\
					(_Timer)->Arg    = (intptr_t)(_Data);											\
					(_Timer)->ToAdd  = (int)(_ToAdd);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)
  #endif

	#define TIMmtx(_Timer, _Mtx, _Expiry, _Period)													\
				do {																				\
					(_Timer)->Ops    = (ABASSI_POST) | (ABASSI_MTX) | (ABASSI_TIM_SRV);				\
					(_Timer)->Ptr    = (void *)(_Mtx);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

	#define TIMsem(_Timer, _Sem, _Expiry, _Period)													\
				do {																				\
					(_Timer)->Ops    = (ABASSI_POST) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Sem);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

#endif

/* ------------------------------------------------	*/
/* Tasks related macros								*/


#define TSKgetPrio(_Tsk)		(((volatile TSK_t *)(_Tsk))->Prio)
#define TSKmyID()				(G_OStaskNow)
#if OX_NAMES
  #define TSKgetID(_Name)		((((_Name)[0]=='\0') ? (G_OStaskNow) : TSKgetID_(_Name)))
#endif


#if OX_PRIO_CHANGE
  #define TSKsetPrio(_Tsk,_New) ((void)Abassi(((ABASSI_NEW_PRIO)|(ABASSI_NEW_O_PRIO)),				\
								              (void *)(_Tsk), (intptr_t)(_New)))
#endif


#if OX_STARVE_WAIT_MAX
  #if (OX_STARVE_PRIO) < 0
	#define TSKgetStrvPrio(_Tsk)			(((volatile TSK_t *)(_Tsk))->StrvPrioMax)
	#define TSKsetStrvPrio(_Tsk, _Prio)		((_Tsk)->StrvPrioMax = (_Prio))
  #else
	#define TSKgetStrvPrio(_Tsk)			(OX_STARVE_PRIO)
  #endif
  #if (OX_STARVE_RUN_MAX) < 0
	#define TSKgetStrvRunMax(_Tsk)			(((volatile TSK_t *)(_Tsk))->StrvRunMax)
	#define TSKsetStrvRunMax(_Tsk, _Cnt)	((_Tsk)->StrvRunMax  = (_Cnt))
  #else
	#define TSKgetStrvRunMax(_Tsk)			(OX_STARVE_RUN_MAX)
  #endif
  #if (OX_STARVE_WAIT_MAX) < 0
	#define TSKgetStrvWaitMax(_Tsk)			(((volatile TSK_t *)(_Tsk))->StrvWaitMax)
	#define TSKsetStrvWaitMax(_Tsk, _Cnt)	((_Tsk)->StrvWaitMax = (_Cnt))
  #else
	#define TSKgetStrvWaitMax(_Tsk)			(OX_STARVE_WAIT_MAX)
  #endif
#else
	#define TSKgetStrvPrio(_Tsk)			(OX_PRIO_MIN)
	#define TSKgetStrvRunMax(_Tsk)			0
	#define TSKgetStrvWaitMax(_Tsk)			0
#endif


#if OX_TASK_SUSPEND
  #define TSKresume(_Tsk)		((void)Abassi((ABASSI_RESUSP),  (void *)(_Tsk),  (intptr_t)0))
  #define TSKsuspend(_Tsk)		((void)Abassi(((ABASSI_RESUSP)|(ABASSI_SUSPEND)), (void *)(_Tsk),	\
								              (intptr_t)-1))
#else
  #define TSKresume(_Tsk)		((void)Abassi((ABASSI_POST), (void *)&((_Tsk)->Priv), (intptr_t)0))
#endif
#define TSKselfSusp()			((void)Abassi((ABASSI_WAIT),(void *)&(G_OStaskNow->Priv),(intptr_t)-1))

#if OX_TIMER_US
  extern void TIMtick(void) REENT_POST;
  #if (OX_ROUND_ROBIN) < 0
	#define TSKgetRR(_Tsk)		(((volatile TSK_t *)(_Tsk))->RRmax)
	#define TSKsetRR(_Tsk,_Tim)	((_Tsk)->RRmax = (_Tim))
  #else
   #if defined(OX_RR_TICK)
	#define TSKgetRR(_Tsk)		(OX_RR_TICK)
   #endif
  #endif
  #if (OX_TIMEOUT) > 0
	#define TSKsleep(_TimeOut)	((void)Abassi((ABASSI_WAIT), (void *)&(G_OStaskNow->Priv),			\
								              (intptr_t)(_TimeOut)))
	#define TSKtimoutKill(_Tsk)	((void)Abassi((ABASSI_TOUT_RM), (void *)(_Tsk), (intptr_t)0))
  #endif
#else
	#define TSKgetRR(_Tsk)		(0U)
#endif

#if OX_COOPERATIVE									/* TSKyeild() is !=  when coopwerative or not	*/
	#define TSKyield()			do { G_OStaskNow->Yield = 1;										\
									(void)Abassi(0U, NULL, (intptr_t)0);							\
								} while(0)
#else
	#define TSKyield()			do {																\
									if (G_OStaskNow->Next != (TSK_t *)NULL) {						\
										(void)Abassi(ABASSI_ADD_RUN, G_OStaskNow, (intptr_t)0);		\
									}																\
								} while(0)
#endif

#if OX_EVENTS
  #define OX_EVT_NOTBLK(_tsk)	(0U == ((volatile TSK_t *)(_tsk))->EventAND)
#else
  #define OX_EVT_NOTBLK(_tsk)	(1)
#endif
#if (OX_TIMEOUT) > 0
  #define OX_TOUT_NOTBLK(_tsk)	((TSK_t *)NULL == ((volatile TSK_t *)(_tsk))->TimPrev)
#else
  #define OX_TOUT_NOTBLK(_tsk)	(1)
#endif

#define TSKstate(_Tsk)			((SEM_t *)NULL == (((volatile TSK_t *)(_Tsk))->Blocker)				\
								? 0								/* 0 is running		*/				\
								: (((((volatile TSK_t *)(_Tsk))->Blocker							\
								                           == (&((volatile TSK_t *)(_Tsk))->Priv))	\
								  && (OX_TOUT_NOTBLK(_Tsk))											\
								  && (OX_EVT_NOTBLK(_Tsk)))											\
								  ? -1			/* -ve is suspended, -ve is suspended */			\
								  :  1))

#define TSKisRdy(_Tsk)			((SEM_t *)NULL == ((volatile TSK_t *)(_Tsk))->Blocker)
#define TSKisBlk(_Tsk)			(TSKstate(_Tsk) > 0)
#define TSKisSusp(_Tsk)			((((volatile TSK_t *)(_Tsk))->Blocker								\
								                          == (&((volatile TSK_t *)(_Tsk))->Priv))	\
								    && (OX_TOUT_NOTBLK(_Tsk))										\
								    && (OX_EVT_NOTBLK(_Tsk)) )

#if OX_USE_TASK_ARG
  #define TSKsetArg(_Tsk, _Arg)	((_Tsk)->TaskArg=(_Arg))
  #define TSKgetArg()			(((volatile TSK_t *)(G_OStaskNow))->TaskArg)		
#endif



/* ------------------------------------------------	*/
/* Logging related macros							*/


#if OX_LOGGING_TYPE
	#define LOG_ENABLE			(0)				/* Enable a specific event number					*/
	#define LOG_DISABLE			(1)				/* Enable a specific event number					*/
	#define LOG_ALL_ON			(2)				/* Enable the recording of all events				*/
	#define LOG_ALL_OFF			(3)				/* Disable the recording of all events				*/
	#define LOG_OFF				(4)				/* Turn off the recording (freeze it)				*/
	#define LOG_ON				(5)				/* Turn on the recording (Continue)					*/ 
	#define LOG_CONT			(6)				/* Start from scratch until OFF/buffer full			*/
	#define LOG_ONCE			(7)				/* Reset the recording: empty the buffer			*/
	#define LOG_DUMP_ALL		(8)				/* Dump all valid events from the buffer			*/
	#define LOG_DUMP_NEXT		(9)				/* Dump the next valid event from the buffer		*/
	#define LOG_GET_NEXT		(10)			/* Retrieve the next valid event from the buffer	*/

	#define LOGallOff()			((void)LOGctrl((LOG_ALL_OFF),   0U))
	#define LOGallOn()			((void)LOGctrl((LOG_ALL_ON),    0U))
	#define LOGdis(_Nmb)		((void)LOGctrl((LOG_DISABLE),  (unsigned int)(_Nmb)))
	#define LOGdumpAll()		((void)LOGctrl((LOG_DUMP_ALL),  0U))
	#define LOGdumpNext()		((void)LOGctrl((LOG_DUMP_NEXT), 0U))
	#define LOGenb(_Nmb)		((void)LOGctrl((LOG_ENABLE),   (unsigned int)(_Nmb)))
	#define LOGgetNext()		      (LOGctrl((LOG_GET_NEXT),  0U))
	#define LOGoff()			((void)LOGctrl((LOG_OFF),       0U))
	#define LOGon()				((void)LOGctrl((LOG_ON),        0U))
	#define LOGreset()			((void)LOGctrl((LOG_RESET),     0U))
	#define LOGstart()			((void)LOGctrl((LOG_START),     0U))

	extern const char   		*LOGctrl(int Ctrl, unsigned int Opt);

#else
	#define LOGallOff()
	#define LOGallOn()
	#define LOGdis(_Nmb)
	#define LOGdumpAll()
	#define LOGdumpNext()
	#define LOGenb(_Nmb)
	#define LOGgetNext()
	#define LOGoff()
	#define LOGon()
	#define LOGreset()
	#define LOGstart()

#endif


#endif											/* The whole file is conditional					*/

/* EOF */
