/* ------------------------------------------------------------------------------------------------ */
/* FILE :		AbassiDemo.h																		*/
/*																									*/
/* CONTENS :																						*/
/*				Contains the build options for the different example code							*/
/*																									*/
/*																									*/
/* Copyright (c) 2011-2012, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*  $Revision: 1.22 $																				*/
/*	$Date: 2012/06/28 15:19:52 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#ifndef _ABASSIDEMO_H_
#define _ABASSIDEMO_H_ 1

/* ------------------------------------------------------------------------------------------------ */
/* Override the definitions in Abassi.h																*/

#define OS_DEF_IN_MAKE 1

/* ------------------------------------------------------------------------------------------------ */
/* Select the task's stack size according to the processor & sometimes tool chain					*/

#if defined (__PIC32MX__)
	#define TC_TSK_STACK	2048


#elif defined(__KEIL__) && defined(__C51__)
	#define TC_TSK_STACK	512


#elif defined(SDCC_z80)
	#define TC_TSK_STACK	256


#elif  defined(__ICC430__)  									\
   || (defined(__GNUC__) && defined(__MSP430__))				\
   || (defined(__TI_COMPILER_VERSION__) && defined(__MSP430__))
  #include "msp430.h"							/* Need to know the device type						*/
  #if defined( __msp430x54x) || defined(__msp430x16x)
	#define TC_TSK_STACK	512
  #else
	#define TC_TSK_STACK	80
  #endif


#elif (defined(__TI_COMPILER_VERSION__) &&defined(__TMS320C28XX__))
	#define TC_TSK_STACK	200


#elif defined(__ICCAVR__)										\
   || (defined(__GNUC__) && defined(__AVR__))
	#define TC_TSK_STACK	100


#elif defined(LPC_P1227) || defined(LPC11U24)
	#define TC_TSK_STACK	384


#elif defined(LPC1343)
	#define TC_TSK_STACK	376


#else
	#define TC_TSK_STACK	1024
#endif


/* ------------------------------------------------------------------------------------------------ */
/* Almost all demos use the same set-up																*/
/* ------------------------------------------------------------------------------------------------ */


#if (OS_DEMO >= 0) && (OS_DEMO <= 7)
	#define OS_ALLOC_SIZE		0				/* When !=0, RTOS supplied OSalloc					*/
	#define OS_COOPERATIVE		0				/* When !=0, the  kernel is in cooperative mode		*/
	#define OS_EVENTS			1				/* If event flags are supported						*/
	#define OS_FCFS				0				/* Allow the use of 1st come 1st serve semaphore	*/
	#define OS_IDLE_STACK		TC_TSK_STACK	/* If IdleTask supplied & if so, stack size			*/
	#define OS_LOGGING_TYPE		0				/* Type of logging to use							*/
	#define OS_MAILBOX			1				/* If mailboxes are used							*/
	#define OS_MAX_PEND_RQST	32				/* Maximum number of sempahores posted in ISRs		*/
	#define OS_MTX_DEADLOCK		0				/* This test validates this							*/
	#define OS_MTX_INVERSION	1				/* To enable protection against priority inversion	*/
	#define OS_NAMES			-1				/* != 0 when named Tasks / Semaphores / Mailboxes	*/
	#define OS_NESTED_INTS		1				/* If operating with nested interrupts				*/
	#define OS_PRIO_CHANGE		1				/* If a task priority can be changed at run time	*/
	#define OS_PRIO_MIN			10				/* Max priority, Idle = OS_PRIO_MIN, AdameEve = 0	*/
	#define OS_PRIO_SAME		1				/* Support multiple tasks with the same priority	*/
	#define OS_ROUND_ROBIN		-1000000		/* Use round-robin, value specifies period in uS	*/
	#define OS_RUNTIME			1				/* If create Task / Semaphore / Mailbox at run time	*/
	#define OS_SEARCH_ALGO		0				/* If using a fast search							*/
	#define OS_STARVE_PRIO		-3				/* Priority threshold for starving protection		*/
	#define OS_STARVE_RUN_MAX	-200			/* Maximum Timer Tick for starving protection		*/
	#define OS_STARVE_WAIT_MAX	-500			/* Maximum time on hold for starving protection		*/
  #if (defined(__TI_COMPILER_VERSION__) &&defined(__TMS320C28XX__))
	#define OS_STATIC_BUF_MBX	0				/* when OS_STATIC_MBOX != 0, # of buffer element	*/
  #else											/* For the C28X we'll use malloc()					*/
	#define OS_STATIC_BUF_MBX	320				/* when OS_STATIC_MBOX != 0, # of buffer element	*/
  #endif
	#define OS_STATIC_MBX		7				/* If !=0 how many mailboxes						*/
	#define OS_STATIC_NAME		0				/* If named mailboxes/semaphore/task, size in char	*/
	#define OS_STATIC_SEM		4				/* If !=0 how many semaphores and mutexes			*/
	#define OS_STATIC_STACK		((TC_TSK_STACK)*(OS_STATIC_TASK))
	#define OS_STATIC_TASK		6				/* If !=0 how many tasks (excluding A&E and Idle)	*/
	#define OS_STATIC_TIM_SRV	0				/* If !=0 how many timer services					*/
	#define OS_TASK_SUSPEND		1				/* If a task can suspend another one				*/
	#define OS_TIMEOUT			1				/* !=0 enables blocking timeout						*/
	#define OS_TIMER_CB			1				/* !=0 gives the timer callback period				*/
	#define OS_TIMER_SRV		0				/* !=0 includes the timer services					*/
	#define OS_TIMER_US			10000			/* !=0 enables timer & specifies the period in uS	*/
	#define OS_USE_TASK_ARG		1				/* If tasks have arguments							*/

  #if defined(LPC_P1227) || defined(LPC11U24)	/* We only have 6 K / 8K, so cut some stuff			*/
	#undef  OS_STATIC_BUF_MBX
	#define OS_STATIC_BUF_MBX 	10
	#undef  OS_STATIC_MBX
	#define	OS_STATIC_MBX		2
  #endif
#endif


/* ------------------------------------------------------------------------------------------------ */
/* IP-USB-FAT Demo settings																			*/
/* ------------------------------------------------------------------------------------------------ */

#if (OS_DEMO == 100)

	#undef  TC_TSK_STACK
	#define TC_TSK_STACK 		2048			/* Dedicated platform demo							*/

	#define OS_ALLOC_SIZE		0				/* When !=0, RTOS supplied OSalloc					*/
	#define OS_COOPERATIVE		0				/* When !=0, the  kernel is in cooperative mode		*/
	#define OS_EVENTS			1				/* If event flags are supported						*/
	#define OS_FCFS				0				/* Allow the use of 1st come 1st serve semaphore	*/
	#define OS_IDLE_STACK		TC_TSK_STACK	/* If IdleTask supplied & if so, stack size			*/
	#define OS_LOGGING_TYPE		0				/* Type of logging to use							*/
	#define OS_MAILBOX			1				/* If mailboxes are used							*/
	#define OS_MAX_PEND_RQST	32				/* Maximum number of sempahores posted in ISRs		*/
	#define OS_MTX_DEADLOCK		0				/* This test validates this							*/
	#define OS_MTX_INVERSION	0				/* To enable protection against priority inversion	*/
	#define OS_NAMES			1				/* != 0 when named Tasks / Semaphores / Mailboxes	*/
	#define OS_NESTED_INTS		1				/* If operating with nested interrupts				*/
	#define OS_PRIO_CHANGE		0				/* If a task priority can be changed at run time	*/
	#define OS_PRIO_MIN			32				/* Max priority, Idle = OS_PRIO_MIN, AdameEve = 0	*/
	#define OS_PRIO_SAME		1				/* Support multiple tasks with the same priority	*/
	#define OS_ROUND_ROBIN		-1000000		/* Use round-robin, value specifies period in uS	*/
	#define OS_RUNTIME			1				/* If create Task / Semaphore / Mailbox at run time	*/
	#define OS_SEARCH_ALGO		0				/* If using a fast search							*/
	#define OS_STARVE_PRIO		0				/* Priority threshold for starving protection		*/
	#define OS_STARVE_RUN_MAX	0				/* Maximum Timer Tick for starving protection		*/
	#define OS_STARVE_WAIT_MAX	0				/* Maximum time on hold for starving protection		*/
	#define OS_STATIC_BUF_MBX	16				/* when OS_STATIC_MBOX != 0, # of buffer element	*/
	#define OS_STATIC_MBX		10				/* If !=0 how many mailboxes						*/
	#define OS_STATIC_NAME		2048			/* If named mailboxes/semaphore/task, size in char	*/
	#define OS_STATIC_SEM		32				/* If !=0 how many semaphores and mutexes			*/
	#define OS_STATIC_STACK		((TC_TSK_STACK)*(OS_STATIC_TASK))
	#define OS_STATIC_TASK		18				/* If !=0 how many tasks (excluding A&E and Idle)	*/
	#define OS_STATIC_TIM_SRV	0				/* If !=0 how many timer services					*/
	#define OS_TASK_SUSPEND		0				/* If a task can suspend another one				*/
	#define OS_TIMEOUT			1				/* !=0 enables blocking timeout						*/
	#define OS_TIMER_CB			0				/* !=0 gives the timer callback period				*/
	#define OS_TIMER_SRV		0				/* !=0 includes the timer services					*/
	#define OS_TIMER_US			10000			/* !=0 enables timer & specifies the period in uS	*/
	#define OS_USE_TASK_ARG		0				/* If tasks have arguments							*/
#endif

#endif

/* EOF */
